@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.h.in,v 1.136 2004/12/14 20:45:22 miguelfreitas Exp $
+ * $Id: xine.h.in,v 1.136.2.3 2005/04/25 14:43:13 mroi Exp $
  *
  * public xine-lib (libxine) interface and documentation
  *
@@ -113,8 +113,8 @@ int  xine_check_version (int major, int minor, int sub) ;
 /* static info - which libxine release this header came from */
 #define XINE_MAJOR_VERSION 1
 #define XINE_MINOR_VERSION 0
-#define XINE_SUB_VERSION   0
-#define XINE_VERSION       "1.0"
+#define XINE_SUB_VERSION   1
+#define XINE_VERSION       "1.0.1"
 
 /*
  * pre-init the xine engine
@@ -992,9 +992,23 @@ char *xine_get_mime_types (xine_t *self);
  * returns NULL if no demuxer is available to handle this. */
 char *xine_get_demux_for_mime_type (xine_t *self, const char *mime_type);
 
-/* get a description string for an input plugin */
+/* get a description string for a plugin */
 const char *xine_get_input_plugin_description (xine_t *self,
 					       const char *plugin_id);
+const char *xine_get_demux_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_spu_plugin_description   (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_audio_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_video_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_audio_driver_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_video_driver_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_post_plugin_description  (xine_t *self,
+					       const char *plugin_id);
 
 /* get lists of available audio and video output plugins */
 const char *const *xine_list_audio_output_plugins (xine_t *self) ;
@@ -1013,6 +1027,8 @@ const char *const *xine_list_spu_plugins(xine_t *self);
 const char *const *xine_list_audio_decoder_plugins(xine_t *self);
 const char *const *xine_list_video_decoder_plugins(xine_t *self);
 
+/* unload unused plugins */
+void xine_plugins_garbage_collector(xine_t *self);
 
 /*********************************************************************
  * visual specific gui <-> xine engine communication                 *
@@ -1474,6 +1490,7 @@ void xine_config_reset (xine_t *self);
 #define XINE_EVENT_VDR_VOLPLUS          329
 #define XINE_EVENT_VDR_VOLMINUS         330
 #define XINE_EVENT_VDR_MUTE             331
+#define XINE_EVENT_VDR_AUDIO            332
 /* some space for further keys */
 #define XINE_EVENT_VDR_SETVIDEOWINDOW   350
 #define XINE_EVENT_VDR_FRAMESIZECHANGED 351
@@ -1822,15 +1839,14 @@ void        xine_osd_draw_line     (xine_osd_t *self, int x1, int y1,
 void        xine_osd_draw_rect     (xine_osd_t *self, int x1, int y1,
 				    int x2, int y2,
 				    int color, int filled );
-/* for freetype2 fonts x1 and y1 specifies the beginning of the baseline,
-   for xine fonts x1 and y1 specifies the upper left corner of the text
-   to be rendered */
+/*  x1 and y1 specifies the upper left corner of the text to be rendered */
 void        xine_osd_draw_text     (xine_osd_t *self, int x1, int y1,
 				    const char *text, int color_base);
 void        xine_osd_draw_bitmap   (xine_osd_t *self, uint8_t *bitmap,
 				    int x1, int y1, int width, int height,
 				    uint8_t *palette_map);
-/* for freetype2 fonts the height is taken from _baseline_ to top */
+/* for freetype2 fonts the height is the maximum height for the whole font and not
+ * only for the specified text */
 void        xine_osd_get_text_size (xine_osd_t *self, const char *text,
 				    int *width, int *height);
 /* with freetype2 support compiled in, you can also specify a font file
@@ -1859,6 +1875,11 @@ void        xine_osd_clear         (xine_osd_t *self);
  * color index, up to the size of the text palette.
  *
  * Use OSD_TEXT1, OSD_TEXT2, ... for some preassigned color indices.
+ *
+ * these palettes are not really well working the true type fonts.
+ * First thing is that these fonts can not have a border. So the best
+ * result you can get by loading a linearly blending palette from the
+ * background (at index 0) to the forground color (at index 10)
  */
 void        xine_osd_set_text_palette (xine_osd_t *self,
 				       int palette_number,
