@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2001 the xine project
+ * Copyright (C) 2001-2003 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_sun_out.c,v 1.20 2002/06/12 12:22:28 f1rmb Exp $
+ * $Id: audio_sun_out.c,v 1.40 2004/12/12 22:01:02 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +32,9 @@
 #include <math.h>
 #include <unistd.h>
 #include <sys/audioio.h>
+#if	HAVE_SYS_MIXER_H
+#include <sys/mixer.h>
+#endif
 #include <sys/uio.h>
 #include <sys/ioctl.h>
 #include <inttypes.h>
@@ -56,16 +59,25 @@
 #define	AUDIO_PRECISION_16	16
 #endif
 
-#define AO_SUN_IFACE_VERSION 4
+#define AO_SUN_IFACE_VERSION 8
 
 #define GAP_TOLERANCE         5000
 #define GAP_NONRT_TOLERANCE   AO_MAX_GAP
 #define	NOT_REAL_TIME		-1
 
 
+typedef struct {
+  audio_driver_class_t	driver_class;
+
+  xine_t	       *xine;
+} sun_class_t;
+
+
 typedef struct sun_driver_s {
 
-  ao_driver_t	 ao_driver;
+  ao_driver_t    ao_driver;
+
+  xine_t	*xine;
 
   char		*audio_dev;
   int            audio_fd;
@@ -114,7 +126,7 @@ typedef struct sun_driver_s {
  * try to figure out, if the soundcard driver provides usable (precise)
  * sample counter information
  */
-static int realtime_samplecounter_available(char *dev)
+static int realtime_samplecounter_available(xine_t *xine, char *dev)
 {
   int fd = -1;
   audio_info_t info;
@@ -137,9 +149,12 @@ static int realtime_samplecounter_available(char *dev)
   if (silence == NULL)
     goto error;
     
-  if ((fd = open(dev, O_WRONLY)) < 0)
+  if ((fd = open(dev, O_WRONLY|O_NONBLOCK)) < 0)
     goto error;
 
+  /* We wanted non blocking open but now put it back to normal */
+  fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) & ~O_NONBLOCK);
+
   AUDIO_INITINFO(&info);
   info.play.sample_rate = 44100;
   info.play.channels = AUDIO_CHANNELS_STEREO;
@@ -147,17 +162,17 @@ static int realtime_samplecounter_available(char *dev)
   info.play.encoding = AUDIO_ENCODING_LINEAR;
   info.play.samples = 0;
   if (ioctl(fd, AUDIO_SETINFO, &info)) {
-    fprintf(stderr, "rtsc: SETINFO failed\n");
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: SETINFO failed\n");
     goto error;
   }
     
   if (write(fd, silence, len) != len) {
-    fprintf(stderr, "rtsc: write failed\n");
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: write failed\n");
     goto error;
   }
 
   if (ioctl(fd, AUDIO_GETINFO, &info)) {
-    fprintf(stderr, "rtsc: GETINFO1, %s\n", strerror(errno));
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: GETINFO1, %s\n", strerror(errno));
     goto error;
   }
 
@@ -179,11 +194,11 @@ static int realtime_samplecounter_available(char *dev)
       break;
 
     if (ioctl(fd, AUDIO_GETINFO, &info)) {
-	fprintf(stderr, "rtsc: GETINFO2 failed, %s\n", strerror(errno));
+	xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: GETINFO2 failed, %s\n", strerror(errno));
 	goto error;
     }
     if (info.play.samples < last_samplecnt) {
-	fprintf(stderr, "rtsc: %u > %u?\n", last_samplecnt, info.play.samples);
+	xprintf(xine, XINE_VERBOSITY_DEBUG, "rtsc: %u > %u?\n", last_samplecnt, info.play.samples);
 	goto error;
     }
 
@@ -234,6 +249,149 @@ error:
 }
 
 
+/* 
+ * match the requested sample rate |sample_rate| against the
+ * sample rates supported by the audio device |dev|.  Return
+ * a supported sample rate,  it that sample rate is close to
+ * (< 1% difference) the requested rate; return 0 otherwise.
+ */
+static int
+find_close_samplerate_match(int dev, int sample_rate)
+{
+#if	HAVE_SYS_MIXER_H
+    am_sample_rates_t *sr;
+    int i, num, err, best_err, best_rate;
+
+    for (num = 16; num < 1024; num *= 2) {
+	sr = malloc(AUDIO_MIXER_SAMP_RATES_STRUCT_SIZE(num));
+	if (!sr)
+	    return 0;
+	sr->type = AUDIO_PLAY;
+	sr->flags = 0;
+	sr->num_samp_rates = num;
+	if (ioctl(dev, AUDIO_MIXER_GET_SAMPLE_RATES, sr)) {
+	    free(sr);
+	    return 0;
+	}
+	if (sr->num_samp_rates <= num)
+	    break;
+	free(sr);
+    }
+
+    if (sr->flags & MIXER_SR_LIMITS) {
+	/*
+	 * HW can playback any rate between 
+	 * sr->samp_rates[0] .. sr->samp_rates[1]
+	 */
+	free(sr);
+	return 0;
+    } else {
+	/* HW supports fixed sample rates only */
+
+	best_err = 65535;
+	best_rate = 0;
+
+	for (i = 0; i < sr->num_samp_rates; i++) {
+	    err = abs(sr->samp_rates[i] - sample_rate);
+	    if (err == 0) {
+		/*
+		 * exact supported sample rate match, no need to
+		 * retry something else
+		 */
+		best_rate = 0;
+		break;
+	    }
+	    if (err < best_err) {
+		best_err = err;
+		best_rate = sr->samp_rates[i];
+	    }
+	}
+
+	free(sr);
+
+	if (best_rate > 0 && 100*best_err < sample_rate) {
+	    /* found a supported sample rate with <1% error? */
+	    return best_rate;
+	}
+	return 0;
+    }
+
+#else
+    int i, err;
+    int audiocs_rates[] = {
+	5510, 6620, 8000, 9600, 11025, 16000, 18900, 22050,
+	27420, 32000, 33075, 37800, 44100, 48000, 0
+    };
+
+    for (i = 0; audiocs_rates[i]; i++) {
+	err = abs(audiocs_rates[i] - sample_rate);
+	if (err == 0) {
+	    /* 
+	     * exact supported sample rate match, no need to
+	     * retry something elise
+	     */
+	    return 0;
+	}
+	if (100*err < audiocs_rates[i]) {
+	    /* <1% error? */
+	    return audiocs_rates[i];
+	}
+    }
+
+    return 0;
+#endif
+}
+
+
+/*
+ * return the highest sample rate supported by audio device |dev|.
+ */
+static int
+find_highest_samplerate(int dev)
+{
+#if	HAVE_SYS_MIXER_H
+    am_sample_rates_t *sr;
+    int i, num, max_rate;
+
+    for (num = 16; num < 1024; num *= 2) {
+	sr = malloc(AUDIO_MIXER_SAMP_RATES_STRUCT_SIZE(num));
+	if (!sr)
+	    return 0;
+	sr->type = AUDIO_PLAY;
+	sr->flags = 0;
+	sr->num_samp_rates = num;
+	if (ioctl(dev, AUDIO_MIXER_GET_SAMPLE_RATES, sr)) {
+	    free(sr);
+	    return 0;
+	}
+	if (sr->num_samp_rates <= num)
+	    break;
+	free(sr);
+    }
+
+    if (sr->flags & MIXER_SR_LIMITS) {
+	/*
+	 * HW can playback any rate between 
+	 * sr->samp_rates[0] .. sr->samp_rates[1]
+	 */
+	max_rate = sr->samp_rates[1];
+    } else {
+	/* HW supports fixed sample rates only */
+	max_rate = 0;
+	for (i = 0; i < sr->num_samp_rates; i++) {
+	    if (sr->samp_rates[i] > max_rate)
+		max_rate = sr->samp_rates[i];
+	}
+    }
+    free(sr);
+    return max_rate;
+
+#else
+    return 44100;	/* should be supported even on old ISA SB cards */
+#endif
+}
+
+
 /*
  * open the audio device for writing to
  *
@@ -251,12 +409,13 @@ static int ao_sun_open(ao_driver_t *this_gen,
 {
   sun_driver_t *this = (sun_driver_t *) this_gen;
   audio_info_t info;
+  int pass;
   int ok;
 
-  printf ("audio_sun_out: ao_sun_open rate=%d, mode=%d\n", rate, mode);
+  xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: ao_sun_open rate=%d, mode=%d\n", rate, mode);
 
   if ( (mode & this->capabilities) == 0 ) {
-    printf ("audio_sun_out: unsupported mode %08x\n", mode);
+    xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: unsupported mode %08x\n", mode);
     return 0;
   }
 
@@ -276,49 +435,88 @@ static int ao_sun_open(ao_driver_t *this_gen,
    * open audio device
    */
 
-  this->audio_fd=open(this->audio_dev,O_WRONLY|O_NONBLOCK);
+  this->audio_fd = open(this->audio_dev, O_WRONLY|O_NONBLOCK);
   if(this->audio_fd < 0) {
-    printf("audio_sun_out: Opening audio device %s: %s\n",
-	   this->audio_dev, strerror(errno));
+    xprintf(this->xine, XINE_VERBOSITY_LOG, 
+	    _("audio_sun_out: opening audio device %s failed: %s\n"), this->audio_dev, strerror(errno));
     return 0;
   }
   
   /* We wanted non blocking open but now put it back to normal */
-  fcntl(this->audio_fd, F_SETFL, fcntl(this->audio_fd, F_GETFL)&~O_NONBLOCK);
+  fcntl(this->audio_fd, F_SETFL, fcntl(this->audio_fd, F_GETFL) & ~O_NONBLOCK);
 
   /*
    * configure audio device
    */
+  for (ok = pass = 0; pass <= 5; pass++) {
 
-  AUDIO_INITINFO(&info);
-  info.play.channels = (mode & AO_CAP_MODE_STEREO)
-      ? AUDIO_CHANNELS_STEREO
-      : AUDIO_CHANNELS_MONO;
-  info.play.precision = bits;
-  info.play.encoding = bits == 8
-      ? AUDIO_ENCODING_LINEAR8
-      : AUDIO_ENCODING_LINEAR;
-  info.play.sample_rate = this->input_sample_rate;
-  info.play.eof = 0;
-  info.play.samples = 0;
+      AUDIO_INITINFO(&info);
+      info.play.channels = (mode & AO_CAP_MODE_STEREO)
+	  ? AUDIO_CHANNELS_STEREO
+	  : AUDIO_CHANNELS_MONO;
+      info.play.precision = bits;
+      info.play.encoding = bits == 8
+	  ? AUDIO_ENCODING_LINEAR8
+	  : AUDIO_ENCODING_LINEAR;
+      info.play.sample_rate = this->input_sample_rate;
+      info.play.eof = 0;
+      info.play.samples = 0;
+
+      this->convert_u8_s8 = 0;
+
+      if (pass & 1) {
+	  /*
+	   * on some sun audio drivers, 8-bit unsigned LINEAR8 encoding is 
+	   * not supported, but 8-bit signed encoding is.
+	   *
+	   * Try S8, and if it works, use our own U8->S8 conversion before
+	   * sending the samples to the sound driver.
+	   */
+	  if (info.play.encoding != AUDIO_ENCODING_LINEAR8)
+	      continue;
+	  info.play.encoding = AUDIO_ENCODING_LINEAR;
+	  this->convert_u8_s8 = 1;
+      }
+
+      if (pass & 2) {
+	  /*
+	   * on some sun audio drivers, only certain fixed sample rates are
+	   * supported.
+	   *
+	   * In case the requested sample rate is very close to one of the
+	   * supported rates,  use the fixed supported rate instead.
+	   *
+	   * XXX: assuming the fixed supported rate works, should we
+	   * lie with our return value and report the requested input
+	   * sample rate, to avoid the software resample code?
+	   */
+	  if (!(info.play.sample_rate =
+		find_close_samplerate_match(this->audio_fd,
+					    this->input_sample_rate))) 
+	      continue;
+      }
+
+      if (pass & 4) {
+	  /* like "pass & 2", but use the highest supported sample rate */
+	  if (!(info.play.sample_rate = find_highest_samplerate(this->audio_fd)))
+	      continue;
+      }
+
+      if ((ok = ioctl(this->audio_fd, AUDIO_SETINFO, &info) >= 0)) {
+	  /* audio format accepted by audio driver */
+	  break;
+      }
 
-  this->convert_u8_s8 = 0;
-  ok = ioctl(this->audio_fd, AUDIO_SETINFO, &info) >= 0;
-  if (!ok && info.play.encoding == AUDIO_ENCODING_LINEAR8) {
       /*
-       * Unsigned AUDIO_ENCODING_LINEAR8 not supported.
-       * Maybe signed AUDIO_ENCODING_LINEAR works?
+       * format not supported?
+       * retry with different encoding and/or sample rate
        */
-      info.play.encoding = AUDIO_ENCODING_LINEAR;
-      ok = ioctl(this->audio_fd, AUDIO_SETINFO, &info) >= 0;
-      if (ok) this->convert_u8_s8 = 1;
   }
 
   if (!ok) {
-      printf("audio_sun_out: Cannot configure audio device for "
-	     "%dhz, %d channel, %d bits\n",
-	     info.play.sample_rate, info.play.channels,
-	     info.play.precision);
+      xprintf(this->xine, XINE_VERBOSITY_DEBUG,
+	      "audio_sun_out: Cannot configure audio device for %dhz, %d channel, %d bits: %s\n",
+	      rate, info.play.channels, bits, strerror(errno));
       close(this->audio_fd);
       this->audio_fd = -1;
       return 0;
@@ -344,7 +542,7 @@ static int ao_sun_open(ao_driver_t *this_gen,
 	   this->input_sample_rate, this->output_sample_rate);
   */
 
-  printf ("audio_sun_out: %d channels output\n",this->num_channels);
+  xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: %d channels output\n",this->num_channels);
   return this->output_sample_rate;
 }
 
@@ -365,11 +563,13 @@ static int ao_sun_delay(ao_driver_t *this_gen)
   sun_driver_t *this = (sun_driver_t *) this_gen;
   audio_info_t info;
 
-  if (ioctl(this->audio_fd, AUDIO_GETINFO, &info) == 0 && info.play.samples) {
+  if (ioctl(this->audio_fd, AUDIO_GETINFO, &info) == 0 &&
+      (this->frames_in_buffer == 0 || info.play.samples > 0)) {
 
     if (info.play.samples < this->last_samplecnt) {
-	printf("*** broken sound driver, sample counter runs backwards, cur %u < prev %u\n",
-	       info.play.samples, this->last_samplecnt);
+	xprintf(this->xine, XINE_VERBOSITY_DEBUG, 
+		"audio_sun_out: broken sound driver, sample counter runs backwards, cur %u < prev %u\n",
+		info.play.samples, this->last_samplecnt);
     }
     this->last_samplecnt = info.play.samples;
 
@@ -495,8 +695,9 @@ static void sun_audio_flush(sun_driver_t *this)
   * I.E. Stereo 16 bits audio frames are 4 bytes.
   */
 static int ao_sun_write(ao_driver_t *this_gen,
-                               int16_t* frame_buffer, uint32_t num_frames)
+			int16_t* data, uint32_t num_frames)
 {
+  uint8_t *frame_buffer=(uint8_t *)data;
   sun_driver_t *this = (sun_driver_t *) this_gen;
   int num_written;
 
@@ -513,9 +714,28 @@ static int ao_sun_write(ao_driver_t *this_gen,
 	  *p ^= 0x80;
   }
   num_written = sun_audio_write(this, frame_buffer, num_frames * this->bytes_per_frame);
-  if (num_written > 0)
+  if (num_written > 0) {
+    int buffered_samples;
+
     this->frames_in_buffer += num_written / this->bytes_per_frame;
 
+    /* 
+     * Avoid storing too much data in the sound driver's buffers.
+     *
+     * When we find more than 3 seconds of buffered audio data in the
+     * driver's buffer, deliberately block sending of more data, until
+     * there is less then 2 seconds of buffered samples.
+     *
+     * During an active audio playback, this helps when either the
+     * xine engine is stopped or a seek operation is performed. In
+     * both cases the buffered audio samples need to be flushed from
+     * the xine engine and the audio driver anyway.
+     */
+    if ((buffered_samples = ao_sun_delay(this_gen)) >= 3*this->output_sample_rate) {
+      sleep(buffered_samples/this->output_sample_rate - 2);
+    }
+  }
+
   return num_written;
 }
 
@@ -552,7 +772,6 @@ static int ao_sun_get_property (ao_driver_t *this_gen, int property) {
 
   switch(property) {
   case AO_PROP_MIXER_VOL:
-    break;
   case AO_PROP_PCM_VOL:
     if (ioctl(this->audio_fd, AUDIO_GETINFO, &info) < 0)
       return 0;
@@ -580,7 +799,6 @@ static int ao_sun_set_property (ao_driver_t *this_gen, int property, int value)
 
   switch(property) {
   case AO_PROP_MIXER_VOL:
-    break;
   case AO_PROP_PCM_VOL:
     info.play.gain = value * AUDIO_MAX_GAIN / 100;
     if (ioctl(this->audio_fd, AUDIO_SETINFO, &info) < 0)
@@ -647,41 +865,53 @@ static int ao_sun_ctrl(ao_driver_t *this_gen, int cmd, ...) {
   return 0;
 }
 
-ao_driver_t *init_audio_out_plugin (config_values_t *config) {
+static ao_driver_t *ao_sun_open_plugin (audio_driver_class_t *class_gen, const void *data) {
 
-  sun_driver_t	  *this;
-  char            *devname;
-  int              audio_fd;
-  int              status;
-  audio_info_t	   info;
+  sun_class_t         *class = (sun_class_t *) class_gen;
+  config_values_t     *config = class->xine->config;
+  sun_driver_t	      *this;
+  char                *devname;
+  char                *audiodev;
+  int                  audio_fd;
+  int                  status;
+  audio_info_t	       info;
 
-  this = (sun_driver_t *) malloc (sizeof (sun_driver_t));
+  this = (sun_driver_t *) xine_xmalloc (sizeof (sun_driver_t));
 
-  /* Fill the .xinerc file with options */ 
+  this->xine = class->xine;
+
+  audiodev = getenv("AUDIODEV");
+
+  /* This config entry is security critical, is it really necessary? */
   devname = config->register_string(config,
-				    "audio.sun_audio_device",
-				    "/dev/audio",
-				    _("device used for audio output with the 'Sun' audio plugin"),
-				    NULL,
-				    NULL,
+				    "audio.device.sun_audio_device",
+				    audiodev && *audiodev ? audiodev : "/dev/audio",
+				    _("Sun audio device name"),
+				    _("Specifies the file name for the Sun audio device "
+				      "to be used.\nThis setting is security critical, "
+				      "because when changed to a different file, xine "
+				      "can be used to fill this file with arbitrary content. "
+				      "So you should be careful that the value you enter "
+				      "really is a proper Sun audio device."),
+				    XINE_CONFIG_SECURITY, NULL,
 				    NULL);
 
   /*
    * find best device driver/channel
    */
 
-  printf ("audio_sun_out: Opening audio device %s...\n", devname);
+  xprintf(this->xine, XINE_VERBOSITY_DEBUG, "audio_sun_out: Opening audio device %s...\n", devname);
 
   /*
    * open the device
    */
 
-  audio_fd=open(this->audio_dev = devname, O_WRONLY|O_NONBLOCK);
+  audio_fd = open(this->audio_dev = devname, O_WRONLY|O_NONBLOCK);
 
   if(audio_fd < 0) 
   {
-    fprintf(stderr, "audio_sun_out: opening audio device %s failed:\n%s\n",
-	   devname, strerror(errno));
+    xprintf(this->xine, XINE_VERBOSITY_LOG,
+	    _("audio_sun_out: opening audio device %s failed: %s\n"), devname, strerror(errno));
 
     free (this);
     return NULL;
@@ -702,23 +932,14 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
    * get capabilities
    */
 
-  this->capabilities = 0;
-
-  printf ("audio_sun_out: supported modes are ");
-
-  this->capabilities |= AO_CAP_MODE_MONO;
-  printf ("mono ");
-
-  this->capabilities |= AO_CAP_MODE_STEREO;
-  printf ("stereo ");
-
-  this->capabilities |= AO_CAP_PCM_VOL | AO_CAP_MUTE_VOL;
-  printf ("\n");
+  this->capabilities = AO_CAP_MODE_MONO | AO_CAP_MODE_STEREO | AO_CAP_8BITS
+  		     | AO_CAP_PCM_VOL | AO_CAP_MUTE_VOL;
 
   close (audio_fd);
 
+  this->xine = class->xine;
   this->audio_fd = -1;
-  this->use_rtsc = realtime_samplecounter_available(this->audio_dev);
+  this->use_rtsc = realtime_samplecounter_available(this->xine, this->audio_dev);
   this->output_sample_rate = 0;
 
   this->ao_driver.get_capabilities	= ao_sun_get_capabilities;
@@ -737,15 +958,51 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   return &this->ao_driver;
 }
 
+/*
+ * class functions
+ */
+
+static char* ao_sun_get_identifier (audio_driver_class_t *this_gen) {
+  return "sun";
+}
+
+static char* ao_sun_get_description (audio_driver_class_t *this_gen) {
+  return _("xine audio output plugin using sun-compliant audio devices/drivers");
+}
+
+static void ao_sun_dispose_class (audio_driver_class_t *this_gen) {
+
+  sun_class_t *this = (sun_class_t *) this_gen;
+
+  free (this);
+}
+
+static void *ao_sun_init_class (xine_t *xine, void *data) {
+  sun_class_t         *this;
+
+  this = (sun_class_t *) xine_xmalloc (sizeof (sun_class_t));
+
+  this->driver_class.open_plugin     = ao_sun_open_plugin;
+  this->driver_class.get_identifier  = ao_sun_get_identifier;
+  this->driver_class.get_description = ao_sun_get_description;
+  this->driver_class.dispose         = ao_sun_dispose_class;
+
+  this->xine = xine;
+
+  return this;
+}
+
+
 static ao_info_t ao_info_sun = {
-  AO_SUN_IFACE_VERSION,
-  "sun",
-  NULL,
   10
 };
 
-ao_info_t *get_audio_out_plugin_info() {
-  ao_info_sun.description = _("xine audio output plugin using sun-compliant audio devices/drivers"); 
-  return &ao_info_sun;
-}
+/*
+ * exported plugin catalog entry
+ */
 
+plugin_info_t xine_plugin_info[] = {
+  /* type, API, "name", version, special_info, init_function */  
+  { PLUGIN_AUDIO_OUT, AO_SUN_IFACE_VERSION, "sun", XINE_VERSION_CODE, &ao_info_sun, ao_sun_init_class },
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+};
