@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.52 2002/07/05 17:32:02 mroi Exp $
+ * $Id: input_file.c,v 1.98.2.1 2005/04/20 17:21:07 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,35 +34,305 @@
 #include <string.h>
 #include <errno.h>
 
+#define LOG_MODULE "input_file"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "compat.h"
 #include "input_plugin.h"
 
-extern int errno;
-
 #define MAXFILES      65535
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
-    printf(__VA_ARGS__);                                             \
-  }
+#ifndef WIN32
+/* MS needs O_BINARY to open files, for everyone else,
+ * make sure it doesn't get in the way */
+#  define O_BINARY  0
 #endif
 
+typedef struct {
+
+  input_class_t     input_class;
+
+  xine_t           *xine;
+  config_values_t  *config;
+  
+  int               show_hidden_files;
+  char             *origin_path;
+
+  int               mrls_allocated_entries;
+  xine_mrl_t      **mrls;
+  
+} file_input_class_t;
+
+typedef struct {
+  input_plugin_t    input_plugin;
+  
+  xine_stream_t    *stream;
+
+  int               fh;
+  char             *mrl;
+
+} file_input_plugin_t;
+
+
+static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
+
+  struct stat          buf ;
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  if (this->fh <0)
+    return 0;
+
+#ifdef _MSC_VER
+    /*return INPUT_CAP_SEEKABLE | INPUT_CAP_GET_DIR;*/
+	return INPUT_CAP_SEEKABLE;
+#else
+  if (fstat (this->fh, &buf) == 0) {
+    if (S_ISREG(buf.st_mode))
+      return INPUT_CAP_SEEKABLE;
+    else
+      return 0;
+  } else
+    perror ("system call fstat");
+  return 0;
+#endif /* _MSC_VER */
+}
+
+
+static off_t file_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  return read (this->fh, buf, len);
+}
+
+static buf_element_t *file_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
+
+  off_t                 num_bytes, total_bytes;
+  file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
+  buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
+
+  buf->content = buf->mem;
+  buf->type = BUF_DEMUX_BLOCK;
+  total_bytes = 0;
+
+  while (total_bytes < todo) {
+    num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
+    if (num_bytes <= 0) {
+      if (num_bytes < 0) {
+	xine_log (this->stream->xine, XINE_LOG_MSG,
+		  _("input_file: read error (%s)\n"), strerror(errno));
+	_x_message(this->stream, XINE_MSG_READ_ERROR,
+                     this->mrl, NULL);
+      }
+      buf->free_buffer (buf);
+      buf = NULL;
+      break;
+    }
+    total_bytes += num_bytes;
+  }
+
+  if( buf != NULL )
+    buf->size = total_bytes;
+
+  return buf;
+}
+
+static off_t file_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  return lseek (this->fh, offset, origin);
+}
+
+static off_t file_plugin_get_current_pos (input_plugin_t *this_gen){
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  if (this->fh <0)
+    return 0;
+
+  return lseek (this->fh, 0, SEEK_CUR);
+}
+
+static off_t file_plugin_get_length (input_plugin_t *this_gen) {
+
+  struct stat          buf ;
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  if (this->fh <0)
+    return 0;
+
+  if (fstat (this->fh, &buf) == 0) {
+    return buf.st_size;
+  } else
+    perror ("system call fstat");
+  return 0;
+}
+
+static uint32_t file_plugin_get_blocksize (input_plugin_t *this_gen) {
+  return 0;
+}
+
+/*
+ * Return 1 is filepathname is a directory, otherwise 0
+ */
+static int is_a_dir(char *filepathname) {
+  struct stat  pstat;
+  
+  stat(filepathname, &pstat);
+
+  return (S_ISDIR(pstat.st_mode));
+}
+
+static char* file_plugin_get_mrl (input_plugin_t *this_gen) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  return this->mrl;
+}
+
+static int file_plugin_get_optional_data (input_plugin_t *this_gen, 
+					  void *data, int data_type) {
+  
+  return INPUT_OPTIONAL_UNSUPPORTED;
+}
+
+static void file_plugin_dispose (input_plugin_t *this_gen ) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  if (this->fh != -1)
+    close(this->fh);
+
+  free (this->mrl);
+
+  free (this);
+}
+
+static char *decode_uri (char *uri) {
+
+  int len = strlen (uri);
+  int i;
+  
+  uri = strdup(uri);
+
+  for (i=0; i<len; i++) {
+
+    if ( (uri[i]=='%') && (i<(len-2)) ) {
+
+      int c;
+
+      if ( sscanf (&uri[i+1], "%02x", &c) == 1) {
+
+	uri[i]= (char) c;
+
+	memmove (uri+i+1, uri+i+3, len-i-3);
+
+	len-=2;
+      }
+    }
+  }
+  
+  uri[len] = 0;
+
+  return uri;
+}
+
+static int file_plugin_open (input_plugin_t *this_gen ) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+  char                *filename;
+
+  lprintf("file_plugin_open\n");
+
+  if (strncasecmp (this->mrl, "file:", 5) == 0)
+  {
+    if (strncasecmp (this->mrl, "file://localhost/", 16) == 0)
+      filename = decode_uri(&(this->mrl[16]));
+    else if (strncasecmp (this->mrl, "file://127.0.0.1/", 16) == 0)
+      filename = decode_uri(&(this->mrl[16]));
+    else
+      filename = decode_uri(&(this->mrl[5]));
+  }
+  else
+    filename = decode_uri(this->mrl);
+
+  this->fh = open (filename, O_RDONLY|O_BINARY);
+  
+  free(filename);
+
+  if (this->fh == -1) {
+    /* try again without unescaping; such MRLs might be invalid,
+     * but we are a nice software */
+    if (strncasecmp (this->mrl, "file:", 5) == 0)
+    {
+      if (strncasecmp (this->mrl, "file://localhost/", 16) == 0)
+        this->fh = open(&this->mrl[16], O_RDONLY|O_BINARY);
+      else if (strncasecmp (this->mrl, "file://127.0.0.1/", 16) == 0)
+        this->fh = open(&this->mrl[16], O_RDONLY|O_BINARY);
+      else
+        this->fh = open(&this->mrl[5], O_RDONLY|O_BINARY);
+    }
+    else
+      this->fh = open(this->mrl, O_RDONLY|O_BINARY);
+    
+    if (this->fh == -1) {
+      if (errno == EACCES) {
+        _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
+	xine_log (this->stream->xine, XINE_LOG_MSG,
+		  _("input_file: Permission denied: >%s<\n"), this->mrl);
+      } else if (errno == ENOENT) {
+        _x_message(this->stream, XINE_MSG_FILE_NOT_FOUND, this->mrl, NULL);
+	xine_log (this->stream->xine, XINE_LOG_MSG,
+		  _("input_file: File not found: >%s<\n"), this->mrl);
+      }
+
+      return 0;
+    }
+  }
+
+  return 1;
+}
+
+static input_plugin_t *file_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
+				    const char *data) {
+
+  /* file_input_class_t  *cls = (file_input_class_t *) cls_gen; */
+  file_input_plugin_t *this;
+  char                *mrl = strdup(data);
+
+  lprintf("file_class_get_instance\n");
+
+  if ((strncasecmp (mrl, "file:", 5)) && strstr (mrl, ":/") && (strstr (mrl, ":/") < strchr(mrl, '/'))) {
+    free (mrl);
+    return NULL;
+  }
+
+  this = (file_input_plugin_t *) xine_xmalloc (sizeof (file_input_plugin_t));
+  this->stream = stream;
+  this->mrl    = mrl;
+  this->fh     = -1;
+
+  this->input_plugin.open               = file_plugin_open;
+  this->input_plugin.get_capabilities   = file_plugin_get_capabilities;
+  this->input_plugin.read               = file_plugin_read;
+  this->input_plugin.read_block         = file_plugin_read_block;
+  this->input_plugin.seek               = file_plugin_seek;
+  this->input_plugin.get_current_pos    = file_plugin_get_current_pos;
+  this->input_plugin.get_length         = file_plugin_get_length;
+  this->input_plugin.get_blocksize      = file_plugin_get_blocksize;
+  this->input_plugin.get_mrl            = file_plugin_get_mrl;
+  this->input_plugin.get_optional_data  = file_plugin_get_optional_data;
+  this->input_plugin.dispose            = file_plugin_dispose;
+  this->input_plugin.input_class        = cls_gen;
+
+  return &this->input_plugin;
+}
+
+
+/*
+ * plugin class functions
+ */
+
 #ifndef S_ISLNK
 #define S_ISLNK(mode)  0
 #endif
@@ -85,38 +355,16 @@ extern int errno;
 #define S_IXUGO        (S_IXUSR | S_IXGRP | S_IXOTH)
 #endif
 
-typedef struct {
-  input_plugin_t    input_plugin;
-
-  xine_t           *xine;
-  
-  int               fh;
-  int               show_hidden_files;
-  char             *origin_path;
-  FILE             *sub;
-  char             *mrl;
-  config_values_t  *config;
-
-  int               mrls_allocated_entries;
-  mrl_t           **mrls;
-  
-} file_input_plugin_t;
-
-
-/* ***************************************************************************
- *                            PRIVATES FUNCTIONS
- */
-
 /*
  * Callback for config changes.
  */
-static void hidden_bool_cb(void *data, cfg_entry_t *cfg) {
-  file_input_plugin_t *this = (file_input_plugin_t *) data;
+static void hidden_bool_cb(void *data, xine_cfg_entry_t *cfg) {
+  file_input_class_t *this = (file_input_class_t *) data;
   
   this->show_hidden_files = cfg->num_value;
 }
-static void origin_change_cb(void *data, cfg_entry_t *cfg) {
-  file_input_plugin_t *this = (file_input_plugin_t *) data;
+static void origin_change_cb(void *data, xine_cfg_entry_t *cfg) {
+  file_input_class_t *this = (file_input_class_t *) data;
   
   this->origin_path = cfg->str_value;
 }
@@ -131,7 +379,7 @@ static void origin_change_cb(void *data, cfg_entry_t *cfg) {
 #define CMP          2
 #define LEN          3
 #define ISDIGIT(c)   ((unsigned) (c) - '0' <= 9)
-static int strverscmp(const char *s1, const char *s2) {
+static int _strverscmp(const char *s1, const char *s2) {
   const unsigned char *p1 = (const unsigned char *) s1;
   const unsigned char *p2 = (const unsigned char *) s2;
   unsigned char c1, c2;
@@ -188,10 +436,10 @@ static int strverscmp(const char *s1, const char *s2) {
 }
 
 /*
- * Wrapper to strverscmp() for qsort() calls, which sort mrl_t type array.
+ * Wrapper to _strverscmp() for qsort() calls, which sort mrl_t type array.
  */
-static int _sortfiles_default(const mrl_t *s1, const mrl_t *s2) {
-  return(strverscmp(s1->mrl, s2->mrl));
+static int _sortfiles_default(const xine_mrl_t *s1, const xine_mrl_t *s2) {
+  return(_strverscmp(s1->mrl, s2->mrl));
 }
 
 /*
@@ -204,9 +452,9 @@ static uint32_t get_file_type(char *filepathname, char *origin, xine_t *xine) {
   char         buf[XINE_PATH_MAX + XINE_NAME_MAX + 1];
 
   if((lstat(filepathname, &pstat)) < 0) {
-    sprintf(buf, "%s/%s", origin, filepathname);
+    snprintf(buf, sizeof(buf), "%s/%s", origin, filepathname);
     if((lstat(buf, &pstat)) < 0) {
-      LOG_MSG(xine, _("lstat failed for %s{%s}\n"), filepathname, origin);
+      lprintf ("lstat failed for %s{%s}\n", filepathname, origin);
       file_type |= mrl_unknown;
       return file_type;
     }
@@ -250,189 +498,31 @@ static off_t get_file_size(char *filepathname, char *origin) {
   char         buf[XINE_PATH_MAX + XINE_NAME_MAX + 1];
 
   if((lstat(filepathname, &pstat)) < 0) {
-    sprintf(buf, "%s/%s", origin, filepathname);
+    snprintf(buf, sizeof(buf), "%s/%s", origin, filepathname);
     if((lstat(buf, &pstat)) < 0)
       return (off_t) 0;
   }
 
   return pstat.st_size;
 }
-/*
- *                              END OF PRIVATES
- *****************************************************************************/
 
-/*
- *
- */
-static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
-
-  return INPUT_CAP_SEEKABLE | INPUT_CAP_PREVIEW | INPUT_CAP_GET_DIR | INPUT_CAP_SPULANG;
+static char *file_class_get_description (input_class_t *this_gen) {
+  return _("file input plugin");
 }
 
-/*
- *
- */
-static int file_plugin_open (input_plugin_t *this_gen, char *mrl) {
-
-  char                *filename, *subtitle;
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  if (this->mrl)
-    free (this->mrl);
-
-  this->mrl = strdup(mrl);
-
-  if (!strncasecmp (this->mrl, "file://", 7))
-    filename = &this->mrl[7];
-  else
-    filename = this->mrl;
-
-  subtitle = strrchr (filename, '%');
-  if (subtitle) {
-    *subtitle = 0;
-    subtitle++;
-
-    LOG_MSG(this->xine, _("input_file: trying to open subtitle file '%s'\n"),
-	    subtitle);
-
-    this->sub = fopen (subtitle, "r");
-
-  } else
-    this->sub = NULL;
-
-
-  this->fh = open (filename, O_RDONLY);
-
-  if (this->fh == -1) {
-    return 0;
-  }
-
-  return 1;
+static const char *file_class_get_identifier (input_class_t *this_gen) {
+  return "file";
 }
 
-/*
- *
- */
-static off_t file_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+static xine_mrl_t **file_class_get_dir (input_class_t *this_gen, 
+					const char *filename, int *nFiles) {
 
-  return read (this->fh, buf, len);
-}
+  /* FIXME: this code needs cleanup badly */
 
-/*
- * helper function to release buffer
- * in case demux thread is cancelled
- */
-static void pool_release_buffer (void *arg) {
-  buf_element_t *buf = (buf_element_t *) arg;
-  if( buf != NULL )
-    buf->free_buffer(buf);
-}
-
-/*
- *
- */
-static buf_element_t *file_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
-
-  off_t                 num_bytes, total_bytes;
-  file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
-  buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
-
-  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
-  pthread_cleanup_push( pool_release_buffer, buf );
-
-  buf->content = buf->mem;
-  buf->type = BUF_DEMUX_BLOCK;
-  total_bytes = 0;
-
-  while (total_bytes < todo) {
-    pthread_testcancel();
-    num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
-    if (num_bytes <= 0) {
-      if (num_bytes < 0) 
-	LOG_MSG_STDERR(this->xine, _("input_file: read error (%s)\n"), strerror(errno));
-      buf->free_buffer (buf);
-      buf = NULL;
-      break;
-    }
-    total_bytes += num_bytes;
-  }
-
-  if( buf != NULL )
-    buf->size = total_bytes;
-
-  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
-  pthread_cleanup_pop(0);
-
-  return buf;
-}
-
-/*
- *
- */
-static off_t file_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  return lseek (this->fh, offset, origin);
-}
-
-/*
- *
- */
-static off_t file_plugin_get_current_pos (input_plugin_t *this_gen){
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  if (this->fh <0)
-    return 0;
-
-  return lseek (this->fh, 0, SEEK_CUR);
-}
-
-/*
- *
- */
-static off_t file_plugin_get_length (input_plugin_t *this_gen) {
-
-  struct stat          buf ;
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  if (this->fh <0)
-    return 0;
-
-  if (fstat (this->fh, &buf) == 0) {
-    return buf.st_size;
-  } else
-    perror ("system call fstat");
-  return 0;
-}
-
-/*
- *
- */
-static uint32_t file_plugin_get_blocksize (input_plugin_t *this_gen) {
-  return 0;
-}
-
-/*
- * Return 1 is filepathname is a directory, otherwise 0
- */
-static int is_a_dir(char *filepathname) {
-  struct stat  pstat;
-  
-  stat(filepathname, &pstat);
-
-  return (S_ISDIR(pstat.st_mode));
-}
-
-/*
- *
- */
-static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen, 
-				    char *filename, int *nFiles) {
-  file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
+  file_input_class_t   *this = (file_input_class_t *) this_gen;
   struct dirent        *pdirent;
   DIR                  *pdir;
-  mrl_t                *hide_files, *dir_files, *norm_files;
+  xine_mrl_t           *hide_files, *dir_files, *norm_files;
   char                  current_dir[XINE_PATH_MAX + 1];
   char                  current_dir_slashed[XINE_PATH_MAX + 1];
   char                  fullfilename[XINE_PATH_MAX + XINE_NAME_MAX + 1];
@@ -453,21 +543,20 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
     snprintf(current_dir, XINE_PATH_MAX, "%s", this->origin_path);
   }
   else {
-    /* Remove exceed '/' */
-    while((filename[strlen(filename) - 1] == '/') && strlen(filename) > 1)
-      filename[strlen(filename) - 1] = '\0';
-    
     snprintf(current_dir, XINE_PATH_MAX, "%s", filename);
     
+    /* Remove exceed '/' */
+    while((current_dir[strlen(current_dir) - 1] == '/') && strlen(current_dir) > 1)
+      current_dir[strlen(current_dir) - 1] = '\0';
   }
 
   /* Store new origin path */
- __try_again_from_home:
+ try_again_from_home:
   
-  this->config->update_string(this->config, "input.file_origin_path", current_dir);
+  this->config->update_string(this->config, "media.files.origin_path", current_dir);
 
   if(strcasecmp(current_dir, "/"))
-    sprintf(current_dir_slashed, "%s/", current_dir);
+    snprintf(current_dir_slashed, sizeof(current_dir_slashed), "%s/", current_dir);
   else
     sprintf(current_dir_slashed, "/");
   
@@ -480,20 +569,20 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
       /* Try one more time with user homedir */
       snprintf(current_dir, XINE_PATH_MAX, "%s", xine_get_homedir());
       already_tried++;
-      goto __try_again_from_home;
+      goto try_again_from_home;
     }
 
     return NULL;
   }
   
-  dir_files  = (mrl_t *) xine_xmalloc(sizeof(mrl_t) * MAXFILES);
-  hide_files = (mrl_t *) xine_xmalloc(sizeof(mrl_t) * MAXFILES);
-  norm_files = (mrl_t *) xine_xmalloc(sizeof(mrl_t) * MAXFILES);
+  dir_files  = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t) * MAXFILES);
+  hide_files = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t) * MAXFILES);
+  norm_files = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t) * MAXFILES);
   
   while((pdirent = readdir(pdir)) != NULL) {
     
     memset(fullfilename, 0, sizeof(fullfilename));
-    sprintf(fullfilename, "%s/%s", current_dir, pdirent->d_name);
+    snprintf(fullfilename, sizeof(fullfilename), "%s/%s", current_dir, pdirent->d_name);
     
     if(is_a_dir(fullfilename)) {
       
@@ -523,10 +612,9 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
 	  memset(linkbuf, 0, sizeof(linkbuf));
 	  linksize = readlink(fullfilename, linkbuf, XINE_PATH_MAX + XINE_NAME_MAX);
 	  
-	  if(linksize < 0) {
-	    LOG_MSG_STDERR(this->xine, _("%s(%d): readlink() failed: %s\n"), 
-			   __XINE_FUNCTION__, __LINE__, strerror(errno));
-	  }
+	  if(linksize < 0) 
+	    xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		     "input_file: readlink() failed: %s\n", strerror(errno));
 	  else {
 	    dir_files[num_dir_files].link = (char *) xine_xmalloc(linksize + 1);
 	    strncpy(dir_files[num_dir_files].link, linkbuf, linksize);
@@ -563,8 +651,8 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
 	  linksize = readlink(fullfilename, linkbuf, XINE_PATH_MAX + XINE_NAME_MAX);
 	  
 	  if(linksize < 0) {
-	    LOG_MSG_STDERR(this->xine, _("%s(%d): readlink() failed: %s\n"), 
-			   __XINE_FUNCTION__, __LINE__, strerror(errno));
+	    xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		     "input_file: readlink() failed: %s\n", strerror(errno));
 	  }
 	  else {
 	    hide_files[num_hide_files].link = (char *) 
@@ -599,8 +687,8 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
 	linksize = readlink(fullfilename, linkbuf, XINE_PATH_MAX + XINE_NAME_MAX);
 	
 	if(linksize < 0) {
-	  LOG_MSG_STDERR(this->xine, _("%s(%d): readlink() failed: %s\n"), 
-			 __XINE_FUNCTION__, __LINE__, strerror(errno));
+	  xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		   "input_file: readlink() failed: %s\n", strerror(errno));
 	}
 	else {
 	  norm_files[num_norm_files].link = (char *) 
@@ -631,13 +719,13 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
      * Sort arrays
      */
     if(num_dir_files)
-      qsort(dir_files, num_dir_files, sizeof(mrl_t), func);
+      qsort(dir_files, num_dir_files, sizeof(xine_mrl_t), func);
     
     if(num_hide_files)
-      qsort(hide_files, num_hide_files, sizeof(mrl_t), func);
+      qsort(hide_files, num_hide_files, sizeof(xine_mrl_t), func);
     
     if(num_norm_files)
-      qsort(norm_files, num_norm_files, sizeof(mrl_t), func);
+      qsort(norm_files, num_norm_files, sizeof(xine_mrl_t), func);
     
     /*
      * Add directories entries
@@ -646,11 +734,11 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
       
       if(num_files >= this->mrls_allocated_entries) {
 	++this->mrls_allocated_entries;
-	this->mrls = realloc(this->mrls, (this->mrls_allocated_entries+1) * sizeof(mrl_t*));
-	this->mrls[num_files] = (mrl_t *) xine_xmalloc(sizeof(mrl_t));
+	this->mrls = realloc(this->mrls, (this->mrls_allocated_entries+1) * sizeof(xine_mrl_t*));
+	this->mrls[num_files] = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t));
       }
       else
-	memset(this->mrls[num_files], 0, sizeof(mrl_t));
+	memset(this->mrls[num_files], 0, sizeof(xine_mrl_t));
       
       MRL_DUPLICATE(&dir_files[i], this->mrls[num_files]); 
 
@@ -664,11 +752,11 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
       
       if(num_files >= this->mrls_allocated_entries) {
 	++this->mrls_allocated_entries;
-	this->mrls = realloc(this->mrls, (this->mrls_allocated_entries+1) * sizeof(mrl_t*));
-	this->mrls[num_files] = (mrl_t *) xine_xmalloc(sizeof(mrl_t));
+	this->mrls = realloc(this->mrls, (this->mrls_allocated_entries+1) * sizeof(xine_mrl_t*));
+	this->mrls[num_files] = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t));
       }
       else
-	memset(this->mrls[num_files], 0, sizeof(mrl_t));
+	memset(this->mrls[num_files], 0, sizeof(xine_mrl_t));
       
       MRL_DUPLICATE(&hide_files[i], this->mrls[num_files]); 
 
@@ -682,11 +770,11 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
 
       if(num_files >= this->mrls_allocated_entries) {
 	++this->mrls_allocated_entries;
-	this->mrls = realloc(this->mrls, (this->mrls_allocated_entries+1) * sizeof(mrl_t*));
-	this->mrls[num_files] = (mrl_t *) xine_xmalloc(sizeof(mrl_t));
+	this->mrls = realloc(this->mrls, (this->mrls_allocated_entries+1) * sizeof(xine_mrl_t*));
+	this->mrls[num_files] = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t));
       }
       else
-	memset(this->mrls[num_files], 0, sizeof(mrl_t));
+	memset(this->mrls[num_files], 0, sizeof(xine_mrl_t));
 
       MRL_DUPLICATE(&norm_files[i], this->mrls[num_files]); 
 
@@ -744,158 +832,33 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
   return this->mrls;
 }
 
-/*
- *
- */
-static int file_plugin_eject_media (input_plugin_t *this_gen) {
-  return 1; /* doesn't make sense */
-}
-
-/*
- *
- */
-static char* file_plugin_get_mrl (input_plugin_t *this_gen) {
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  return this->mrl;
-}
-
-/*
- *
- */
-static void file_plugin_close (input_plugin_t *this_gen) {
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  close(this->fh);
-  this->fh = -1;
-
-  if (this->sub) {
-    fclose (this->sub);
-    this->sub = NULL;
-  }
-}
-
-/*
- *
- */
-static void file_plugin_stop (input_plugin_t *this_gen) {
-
-  file_plugin_close(this_gen);
-}
-
-/*
- *
- */
-static char *file_plugin_get_description (input_plugin_t *this_gen) {
-  return _("plain file input plugin as shipped with xine");
-}
-
-/*
- *
- */
-static char *file_plugin_get_identifier (input_plugin_t *this_gen) {
-  return "file";
-}
-
-/*
- *
- */
-static int file_plugin_get_optional_data (input_plugin_t *this_gen, 
-					  void *data, int data_type) {
-  
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-#ifdef LOG
-  LOG_MSG(this->xine, _("input_file: get optional data, type %08x, sub %p\n"),
-	  data_type, this->sub);
-#endif
-
-  switch(data_type) {
-  case INPUT_OPTIONAL_DATA_TEXTSPU0:
-    if(this->sub) {
-      FILE **tmp;
-      
-      /* dirty hacks... */
-      tmp = data;
-      *tmp = this->sub;
-      
-      return INPUT_OPTIONAL_SUCCESS;
-    }
-    break;
-    
-  case INPUT_OPTIONAL_DATA_SPULANG:
-    sprintf(data, "%3s", (this->sub) ? "on" : "off");
-    return INPUT_OPTIONAL_SUCCESS;
-    break;
-    
-  default:
-    return INPUT_OPTIONAL_UNSUPPORTED;
-    break;
-
-  }
-
-  return INPUT_OPTIONAL_UNSUPPORTED;
-}
-
-static void file_plugin_dispose (input_plugin_t *this_gen ) {
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-  if (this->mrl)
-    free (this->mrl);
+static void file_class_dispose (input_class_t *this_gen) {
+  file_input_class_t  *this = (file_input_class_t *) this_gen;
 
   free (this->mrls);
   free (this);
 }
 
-/*
- *
- */
-input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
+static void *init_plugin (xine_t *xine, void *data) {
 
-  file_input_plugin_t *this;
+  file_input_class_t  *this;
   config_values_t     *config;
 
-  if (iface != 8) {
-    LOG_MSG(xine,
-	    _("file input plugin doesn't support plugin API version %d.\n"
-	      "PLUGIN DISABLED.\n"
-	      "This means there's a version mismatch between xine and this input"
-	      "plugin.\nInstalling current input plugins should help.\n"),
-	    iface);
-    return NULL;
-  }
+  this = (file_input_class_t *) xine_xmalloc (sizeof (file_input_class_t));
 
-  this       = (file_input_plugin_t *) xine_xmalloc (sizeof (file_input_plugin_t));
-  config     = xine->config;
-  this->xine = xine;
-
-  this->input_plugin.interface_version  = INPUT_PLUGIN_IFACE_VERSION;
-  this->input_plugin.get_capabilities   = file_plugin_get_capabilities;
-  this->input_plugin.open               = file_plugin_open;
-  this->input_plugin.read               = file_plugin_read;
-  this->input_plugin.read_block         = file_plugin_read_block;
-  this->input_plugin.seek               = file_plugin_seek;
-  this->input_plugin.get_current_pos    = file_plugin_get_current_pos;
-  this->input_plugin.get_length         = file_plugin_get_length;
-  this->input_plugin.get_blocksize      = file_plugin_get_blocksize;
-  this->input_plugin.get_dir            = file_plugin_get_dir;
-  this->input_plugin.eject_media        = file_plugin_eject_media;
-  this->input_plugin.get_mrl            = file_plugin_get_mrl;
-  this->input_plugin.close              = file_plugin_close;
-  this->input_plugin.stop               = file_plugin_stop;
-  this->input_plugin.get_description    = file_plugin_get_description;
-  this->input_plugin.get_identifier     = file_plugin_get_identifier;
-  this->input_plugin.get_autoplay_list  = NULL;
-  this->input_plugin.get_optional_data  = file_plugin_get_optional_data;
-  this->input_plugin.dispose            = file_plugin_dispose;
-  this->input_plugin.is_branch_possible = NULL;
-
-  this->fh                     = -1;
-  this->sub                    = NULL;
-  this->mrl                    = NULL;
-  this->config                 = config;
+  this->xine   = xine;
+  this->config = xine->config;
+  config       = xine->config;
   
-  this->mrls = (mrl_t **) xine_xmalloc(sizeof(mrl_t*));
+  this->input_class.get_instance       = file_class_get_instance;
+  this->input_class.get_identifier     = file_class_get_identifier;
+  this->input_class.get_description    = file_class_get_description;
+  this->input_class.get_dir            = file_class_get_dir;
+  this->input_class.get_autoplay_list  = NULL;
+  this->input_class.dispose            = file_class_dispose;
+  this->input_class.eject_media        = NULL;
+
+  this->mrls = (xine_mrl_t **) xine_xmalloc(sizeof(xine_mrl_t*));
   this->mrls_allocated_entries = 0;
 
   {
@@ -904,14 +867,30 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
     if(getcwd(current_dir, sizeof(current_dir)) == NULL)
       strcpy(current_dir, ".");
 
-    this->origin_path = config->register_string(this->config, "input.file_origin_path",
-						current_dir, _("origin path to grab file mrls"),
-						NULL, origin_change_cb, (void *) this);
+    this->origin_path = config->register_string(config, "media.files.origin_path",
+						current_dir, 
+						_("file browsing start location"),
+						_("The browser to select the file to play will "
+						  "start at this location."),
+						0, origin_change_cb, (void *) this);
   }
   
-  this->show_hidden_files = this->config->register_bool(this->config, "input.file_hidden_files", 
-							1, _("hidden files displaying."),
-							NULL, hidden_bool_cb, (void *) this);
+  this->show_hidden_files = config->register_bool(config, 
+						  "media.files.show_hidden_files", 
+						  0, _("list hidden files"),
+						  _("If enabled, the browser to select the file to "
+						    "play will also show hidden files."),
+						  10, hidden_bool_cb, (void *) this);
   
-  return (input_plugin_t *) this;
+  return this;
 }
+
+/*
+ * exported plugin catalog entry
+ */
+
+plugin_info_t xine_plugin_info[] = {
+  /* type, API, "name", version, special_info, init_function */  
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "FILE", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+};
