@@ -19,7 +19,7 @@
  *
  * input plugin for http network streams
  *
- * $Id: input_http.c,v 1.102 2004/12/24 01:59:12 dsalt Exp $
+ * $Id: input_http.c,v 1.102.2.3 2005/04/20 17:21:08 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,7 @@
 #include <string.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <ctype.h>
 
 #ifndef WIN32
 #include <netdb.h>
@@ -99,6 +100,9 @@ typedef struct {
   off_t            shoutcast_pos;
   char            *shoutcast_songtitle;
 
+  /* NSV */
+  int              is_nsv;
+
   /* scratch buffer for forward seeking */
 
   char             seek_buf[BUFSIZE];
@@ -153,8 +157,8 @@ static void no_proxy_list_change_cb(void *this_gen, xine_cfg_entry_t *cfg) {
 }
 
 /*
- * handle no proxy list config option and returns, if use the proxy or not
- * if error occured, is expected using the proxy
+ * handle no-proxy list config option and returns, if use the proxy or not
+ * if error occurred, is expected using the proxy
  */
 static int _x_use_proxy(http_input_class_t *this, const char *host) {
   const char *target;
@@ -176,16 +180,37 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
 
   host_len = strlen(target);
   no_proxy = strdup(this->noproxylist);
-  domain = strtok_r(no_proxy, ", ", &ptr);
+  domain = strtok_r(no_proxy, ",", &ptr);
   while (domain) {
-    noprox_len = strlen(domain);
-    if (host_len >= noprox_len && strcmp(target + host_len - noprox_len, domain) == 0) {
-      lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
-      return 1;
+    /* skip leading spaces */
+    while (isspace (*domain))
+      ++domain;
+    /* only check for matches if we've not reached the end of the token */
+    if (*domain) {
+      /* special-case domain beginning with '=' -> is a host name */
+      if (domain[0] == '=' && strcmp(target, domain + 1) == 0) {
+	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
+	return 1;
+      }
+      noprox_len = strlen(domain);
+      /* special-case host==domain, avoiding dot checks */
+      if (host_len == noprox_len && strcmp(target, domain) == 0) {
+	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
+	return 1;
+      }
+      /* check for host in domain, and require that (if matched) the domain
+       * name is preceded by a dot, either in the host or domain strings,
+       * e.g. "a.foo.bar" is in "foo.bar" and ".foo.bar" but not "o.bar"
+       */
+      if (host_len > noprox_len
+	  && (domain[0] == '.' || target[host_len - noprox_len - 1] == '.')
+	  && strcmp(target + host_len - noprox_len, domain) == 0) {
+	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
+	return 1;
+      }
+      lprintf("host '%s' isn't in no-proxy domain '%s'\n", target, domain);
     }
-    lprintf("host '%s' isn't in no-proxy domain '%s'\n", target, domain);
-    
-    domain = strtok_r(NULL, ", ", &ptr);
+    domain = strtok_r(NULL, ",", &ptr);
     i++;
   }
   free(no_proxy);
@@ -301,7 +326,7 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
             strncat(data.str, songtitle, sizeof(data.str) - len - 3);
           } else {
             strncpy(data.str, songtitle, sizeof(data.str));
-          }  
+          }
           data.str[sizeof(data.str) - 1] = '\0';
           data.str_len = strlen(data.str) + 1;
 
@@ -401,123 +426,49 @@ static off_t http_plugin_read (input_plugin_t *this_gen,
   return num_bytes;
 }
 
-static int read_shoutcast_header(http_input_plugin_t *this) {
-  int done, len, linenum;
+static int resync_nsv(http_input_plugin_t *this) {
+  uint8_t c;
+  int pos = 0;
+  int read_bytes = 0;
 
-  done = 0; len = 0; linenum = 0;
-  while (!done) {
-
-    if (_x_io_tcp_read (this->stream, this->fh, &this->buf[len], 1) != 1) {
-      return 0;
-    }
-
-    if (this->buf[len] == '\012') {
-
-      this->buf[len] = '\0';
-      len--;
-
-      if (len >= 0 && this->buf[len] == '\015') {
-        this->buf[len] = '\0';
-        len--;
-      }
-
-      linenum++;
-
-      lprintf ("shoutcast answer: >%s<\n", this->buf);
-
-      if (!strncasecmp(this->buf, TAG_ICY_NAME, sizeof(TAG_ICY_NAME) - 1)) {
-        _x_meta_info_set(this->stream, XINE_META_INFO_ALBUM,
-                         (this->buf + sizeof(TAG_ICY_NAME) - 1 +
-                          (*(this->buf + sizeof(TAG_ICY_NAME)) == ' ')));
-        _x_meta_info_set(this->stream, XINE_META_INFO_TITLE,
-                         (this->buf + sizeof(TAG_ICY_NAME) - 1 +
-                          (*(this->buf + sizeof(TAG_ICY_NAME)) == ' ')));
-      }
-      
-      if (!strncasecmp(this->buf, TAG_ICY_GENRE, sizeof(TAG_ICY_GENRE) - 1)) {
-        _x_meta_info_set(this->stream, XINE_META_INFO_GENRE,
-                        (this->buf + sizeof(TAG_ICY_GENRE) - 1 +
-                         (*(this->buf + sizeof(TAG_ICY_GENRE)) == ' ')));
-      }
-      
-      /* icy-notice1 is always the same */
-      if (!strncasecmp(this->buf, TAG_ICY_NOTICE2, sizeof(TAG_ICY_NOTICE2) - 1)) {
-        _x_meta_info_set(this->stream, XINE_META_INFO_COMMENT,
-                         (this->buf + sizeof(TAG_ICY_NOTICE2) - 1 +
-                          (*(this->buf + sizeof(TAG_ICY_NOTICE2)) == ' ')));
-      }
-
-      /* metadata interval (in byte) */
-      if (sscanf(this->buf, "icy-metaint:%d", &this->shoutcast_metaint) == 1) {
-        lprintf("shoutcast_metaint: %d\n", this->shoutcast_metaint);
-      }
-
-      /* content type */
-      if (!strncasecmp(this->buf, TAG_CONTENT_TYPE, sizeof(TAG_CONTENT_TYPE) - 1)) {
-        if (!strncasecmp(this->buf + sizeof(TAG_CONTENT_TYPE) - 1, "video/nsv", 9)) {
-          lprintf("shoutcast nsv detected\n");
-          this->shoutcast_mode = 2;
-        } else {
-          lprintf("shoutcast mp3 detected\n");
-        }
-      }
-
-      if (len == -1)
-        done = 1;
-      else
-        len = 0;
-    } else
-      len ++;
-  }
+  lprintf("resyncing NSV stream\n");
+  while ((pos < 3) && (read_bytes < (1024*1024))) {
   
-  this->shoutcast_pos = 0;
-  
-  /* NSV resync */
-  if (this->shoutcast_mode == 2) {
-    uint8_t c;
-    int pos = 0;
-    int read_bytes = 0;
-  
-    lprintf("resyncing NSV stream\n");
-    while ((pos < 3) && (read_bytes < (1024*1024))) {
-    
-      if (http_plugin_read_int(this, &c, 1) != 1)
-        return 1;
+    if (http_plugin_read_int(this, &c, 1) != 1)
+      return 1;
 
-      this->preview[pos] = c;
-      switch (pos) {
-        case 0:
+    this->preview[pos] = c;
+    switch (pos) {
+      case 0:
+        if (c == 'N')
+          pos++;
+        break;
+      case 1:
+        if (c == 'S')
+          pos++;
+        else
+          if (c != 'N')
+            pos = 0;
+        break;
+      case 2:
+        if (c == 'V')
+          pos++;
+        else
           if (c == 'N')
-            pos++;
-          break;
-        case 1:
-          if (c == 'S')
-            pos++;
+            pos = 1;
           else
-            if (c != 'N')
-              pos = 0;
-          break;
-        case 2:
-          if (c == 'V')
-            pos++;
-          else
-            if (c == 'N')
-              pos = 1;
-            else
-              pos = 0;
-          break;
-      }
-      read_bytes++;
-    }
-    if (pos == 3) {
-      lprintf("NSV stream resynced\n");
-    } else {
-      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-        "http: cannot resync NSV stream!\n");
-      return 0;
+            pos = 0;
+        break;
     }
+    read_bytes++;
+  }
+  if (pos == 3) {
+    lprintf("NSV stream resynced\n");
+  } else {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+      "http: cannot resync NSV stream!\n");
+    return 0;
   }
-  lprintf ("end of the shoutcast header\n");
 
   return 1;
 }
@@ -593,7 +544,7 @@ static off_t http_plugin_seek(input_plugin_t *this_gen, off_t offset, int origin
         this->curpos = offset;
       else
         xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-          "http: cannot seek back! (%" PRIiMAX " > %" PRIiMAX ")\n",
+          "http: cannot seek back! (%" PRIdMAX " > %" PRIdMAX ")\n",
           (intmax_t)this->curpos, (intmax_t)offset);
       
     } else {
@@ -675,7 +626,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   http_input_plugin_t *this = (http_input_plugin_t *) this_gen;
   http_input_class_t  *this_class = (http_input_class_t *) this->input_plugin.input_class;
   int                  done, len, linenum;
-  int                  shoutcast = 0, httpcode;
+  int                  httpcode;
   int                  res, progress;
   int                  buflen;
   int                  use_proxy;
@@ -839,15 +790,12 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	if (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
 		   &httpcode, httpstatus) != 4)	{
 	  
-	  /* icecast ? */
+	  /* icecast 1 ? */
 	  if (sscanf(this->buf, "ICY %d %50[^\015\012]", &httpcode, httpstatus) != 2)	{
 	    _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "invalid http answer", NULL);
 	    xine_log (this->stream->xine, XINE_LOG_MSG, 
 		      _("input_http: invalid http answer\n"));
 	    return 0;
-	  } else {
-	    shoutcast = 1;
-	    done = 1;
 	  }
 	}
 
@@ -881,7 +829,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	  
 	  if (sscanf(this->buf, "Content-Length: %" SCNdMAX , &contentlength) == 1) {
       	    xine_log (this->stream->xine, XINE_LOG_MSG, 
-              _("input_http: content length = %" PRIiMAX " bytes\n"),
+              _("input_http: content length = %" PRIdMAX " bytes\n"),
               contentlength);
 	    this->contentlength = (off_t)contentlength;
 	  }
@@ -897,6 +845,44 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
           this->mrl = href;
           return http_plugin_open(this_gen);
         }
+
+        /* Icecast / ShoutCast Stuff */
+        if (!strncasecmp(this->buf, TAG_ICY_NAME, sizeof(TAG_ICY_NAME) - 1)) {
+          _x_meta_info_set(this->stream, XINE_META_INFO_ALBUM,
+                           (this->buf + sizeof(TAG_ICY_NAME) - 1 +
+                            (*(this->buf + sizeof(TAG_ICY_NAME)) == ' ')));
+          _x_meta_info_set(this->stream, XINE_META_INFO_TITLE,
+                           (this->buf + sizeof(TAG_ICY_NAME) - 1 +
+                            (*(this->buf + sizeof(TAG_ICY_NAME)) == ' ')));
+        }
+        
+        if (!strncasecmp(this->buf, TAG_ICY_GENRE, sizeof(TAG_ICY_GENRE) - 1)) {
+          _x_meta_info_set(this->stream, XINE_META_INFO_GENRE,
+                          (this->buf + sizeof(TAG_ICY_GENRE) - 1 +
+                           (*(this->buf + sizeof(TAG_ICY_GENRE)) == ' ')));
+        }
+        
+        /* icy-notice1 is always the same */
+        if (!strncasecmp(this->buf, TAG_ICY_NOTICE2, sizeof(TAG_ICY_NOTICE2) - 1)) {
+          _x_meta_info_set(this->stream, XINE_META_INFO_COMMENT,
+                           (this->buf + sizeof(TAG_ICY_NOTICE2) - 1 +
+                            (*(this->buf + sizeof(TAG_ICY_NOTICE2)) == ' ')));
+        }
+  
+        /* metadata interval (in byte) */
+        if (sscanf(this->buf, TAG_ICY_METAINT"%d", &this->shoutcast_metaint) == 1) {
+          lprintf("shoutcast_metaint: %d\n", this->shoutcast_metaint);
+          this->shoutcast_mode = 1;
+          this->shoutcast_pos = 0;
+        }
+  
+        /* content type */
+        if (!strncasecmp(this->buf, TAG_CONTENT_TYPE, sizeof(TAG_CONTENT_TYPE) - 1)) {
+          if (!strncasecmp(this->buf + sizeof(TAG_CONTENT_TYPE) - 1, "video/nsv", 9)) {
+            lprintf("shoutcast nsv detected\n");
+            this->is_nsv = 1;
+          }
+        }
       }
  
       if (len == -1)
@@ -909,27 +895,14 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 
   lprintf ("end of headers\n");
 
-  
-  /* Trivial shoutcast detection */
-  this->shoutcast_metaint = 0;
-  this->shoutcast_songtitle = NULL;
-  if (shoutcast) {
-    this->shoutcast_mode = 1;
-    if (!read_shoutcast_header(this)) {
-      /* problem when reading shoutcast header */
-      _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "can't read shoutcast header", NULL);
-      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "can't read shoutcast header\n");
-      return 0;
-    }
-  } else {
-    this->shoutcast_mode = 0;
-  }
-
   /*
    * fill preview buffer
    */
   this->preview_size = MAX_PREVIEW_SIZE;
-  if (this->shoutcast_mode == 2) {
+  if (this->is_nsv) {
+    if (!resync_nsv(this))
+      return 0;
+    
     /* the first 3 chars are "NSV" */
     this->preview_size = http_plugin_read_int (this, this->preview + 3, MAX_PREVIEW_SIZE - 3);
   } else {
@@ -1069,8 +1042,8 @@ static void *init_class (xine_t *xine, void *data) {
 						_("The password for the HTTP proxy."), 10,
 						proxy_password_change_cb, (void *) this);
   this->noproxylist = config->register_string(config,
-					      "input.http_no_proxy", "", _("Domains, where to ignore the HTTP proxy"),
-					      _("A Comma separated list of domain names, where the proxy is to be ignored."), 10,
+					      "media.network.http_no_proxy", "", _("Domains for which to ignore the HTTP proxy"),
+					      _("A comma-separated list of domain names for which the proxy is to be ignored.\nIf a domain name is prefixed with '=' then it is treated as a host name only (full match required)."), 10,
 					      no_proxy_list_change_cb, (void *) this);
   
   return this;
@@ -1082,6 +1055,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 15, "http", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "http", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
