@@ -104,6 +104,8 @@ static uint32_t get_bits(uint8_t *buffer, uint32_t count, uint32_t *bit_position
 void mpeg2_header_state_init (picture_t * picture)
 {
     picture->scan = mpeg2_scan_norm;
+    picture->load_intra_quantizer_matrix = 1;
+    picture->load_non_intra_quantizer_matrix = 1;
 }
 
 int mpeg2_header_sequence (picture_t * picture, uint8_t * buffer)
@@ -116,8 +118,11 @@ int mpeg2_header_sequence (picture_t * picture, uint8_t * buffer)
 
     height = (buffer[0] << 16) | (buffer[1] << 8) | buffer[2];
 
-    width = ((height >> 12) + 15) & ~15;
-    height = ((height & 0xfff) + 15) & ~15;
+    picture->display_width = width = (height >> 12);
+    picture->display_height = height = (height & 0xfff);
+    
+    width = (width + 15) & ~15;
+    height = (height + 15) & ~15;
 
     if ((width > 1920) || (height > 1152))
 	return 1;	/* size restrictions for MP@HL */
@@ -147,7 +152,8 @@ int mpeg2_header_sequence (picture_t * picture, uint8_t * buffer)
     else
 	for (i = 0; i < 64; i++)
 	    picture->non_intra_quantizer_matrix[i] = 16;
-
+    picture->load_intra_quantizer_matrix = 1;
+    picture->load_non_intra_quantizer_matrix = 1;
     /* MPEG1 - for testing only */
     picture->mpeg1 = 1;
     picture->intra_dc_precision = 0;
@@ -177,6 +183,7 @@ static int sequence_extension (picture_t * picture, uint8_t * buffer)
 	picture->coded_picture_height =
 	    (picture->coded_picture_height + 31) & ~31;
     
+    
     /* printf ("libmpeg2: low_delay : %d\n", picture->low_delay); */
 
 /*
@@ -184,6 +191,9 @@ static int sequence_extension (picture_t * picture, uint8_t * buffer)
 	    buffer[5], buffer[5] % 0x80);
  */
 
+    picture->frame_rate_ext_n = buffer[5] & 0x31;
+    picture->frame_rate_ext_d = (buffer[5] >> 2) & 0x03;
+    
     /* MPEG1 - for testing only */
     picture->mpeg1 = 0;
 
