@@ -25,16 +25,11 @@
 
 #include <inttypes.h>
 
+#include "xine_internal.h"
 #include "video_out.h"
 #include "mpeg2_internal.h"
 #include "attributes.h"
 
-extern mpeg2_mc_t mpeg2_mc;
-extern void (* mpeg2_idct_copy) (int16_t * block, uint8_t * dest, int stride);
-extern void (* mpeg2_idct_add) (int16_t * block, uint8_t * dest, int stride);
-extern void (* mpeg2_cpu_state_save) (cpu_state_t * state);
-extern void (* mpeg2_cpu_state_restore) (cpu_state_t * state);
-
 #include "vlc.h"
 
 static int non_linear_quantizer_scale [] = {
@@ -992,8 +987,14 @@ static inline void slice_non_intra_DCT (picture_t * picture, uint8_t * dest,
 #define MOTION(table,ref,motion_x,motion_y,size,y)			      \
     pos_x = 2 * picture->offset + motion_x;				      \
     pos_y = 2 * picture->v_offset + motion_y + 2 * y;			      \
-    if ((pos_x > picture->limit_x) || (pos_y > picture->limit_y_ ## size))    \
-	return;								      \
+    if (pos_x > picture->limit_x) {					      \
+      pos_x = ((int)pos_x < 0) ? 0 : picture->limit_x;			      \
+      motion_x = pos_x - 2 * picture->offset;				      \
+    }									      \
+    if (pos_y > picture->limit_y_ ## size){				      \
+      pos_y = ((int)pos_y < 0) ? 0 : picture->limit_y_ ## size;		      \
+      motion_y = pos_y - 2 * picture->v_offset - 2 * y;			      \
+    }									      \
     xy_half = ((pos_y & 1) << 1) | (pos_x & 1);				      \
     table[xy_half] (picture->dest[0] + y * picture->pitches[0] +	      \
 		    picture->offset, ref[0] + (pos_x >> 1) +		      \
@@ -1015,8 +1016,14 @@ static inline void slice_non_intra_DCT (picture_t * picture, uint8_t * dest,
 #define MOTION_FIELD(table,ref,motion_x,motion_y,dest_field,op,src_field)     \
     pos_x = 2 * picture->offset + motion_x;				      \
     pos_y = picture->v_offset + motion_y;				      \
-    if ((pos_x > picture->limit_x) || (pos_y > picture->limit_y))	      \
-	return;								      \
+    if (pos_x > picture->limit_x) {					      \
+      pos_x = ((int)pos_x < 0) ? 0 : picture->limit_x;			      \
+      motion_x = pos_x - 2 * picture->offset;				      \
+    }									      \
+    if (pos_y > picture->limit_y){					      \
+      pos_y = ((int)pos_y < 0) ? 0 : picture->limit_y;			      \
+      motion_y = pos_y - picture->v_offset;				      \
+    }									      \
     xy_half = ((pos_y & 1) << 1) | (pos_x & 1);				      \
     table[xy_half] (picture->dest[0] + dest_field * picture->pitches[0] +     \
 		    picture->offset,					      \
@@ -1176,9 +1183,18 @@ static void motion_fr_dmv (picture_t * picture, motion_t * motion,
     other_y = ((motion_y * m + (motion_y > 0)) >> 1) + dmv_y + 1;
     MOTION_FIELD (mpeg2_mc.put, motion->ref[0], other_x, other_y, 1, & ~1, 0);
 
-    xy_half = ((motion_y & 1) << 1) | (motion_x & 1);
-    offset = (picture->offset + (motion_x >> 1) +
-	      (picture->v_offset + (motion_y & ~1)) * picture->pitches[0]);
+    pos_x = 2 * picture->offset + motion_x;
+    pos_y = picture->v_offset + motion_y;
+    if(pos_x > picture->limit_x){
+      pos_x = ((int)pos_x < 0) ? 0 : picture->limit_x;
+      motion_x = pos_x - 2 * picture->offset;
+    }
+    if(pos_y > picture->limit_y){
+      pos_y = ((int)pos_y < 0) ? 0 : picture->limit_y;
+      motion_y = pos_y - picture->v_offset;
+    }
+    xy_half = ((pos_y & 1) << 1) | (pos_x & 1);
+    offset = (pos_x >> 1) + (pos_y & ~1) * picture->pitches[0];
     mpeg2_mc.avg[xy_half]
 	(picture->dest[0] + picture->offset,
 	 motion->ref[0][0] + offset, 2 * picture->pitches[0], 8);
@@ -1416,29 +1432,27 @@ do {								\
 		  mpeg2_mc.avg : mpeg2_mc.put));		\
 } while (0)
 
-#define NEXT_MACROBLOCK							\
-do {									\
-    picture->offset += 16;						\
-    if (picture->offset == picture->coded_picture_width) {		\
-	do { /* just so we can use the break statement */		\
-	    if (picture->current_frame->copy) {				\
-		picture->current_frame->copy (picture->current_frame,	\
-					      picture->dest);		\
-		if (picture->picture_coding_type == B_TYPE)		\
-		    break;						\
-	    }								\
-	    picture->dest[0] += 16 * picture->pitches[0];		\
-	    picture->dest[1] += 8 * picture->pitches[1];		\
-	    picture->dest[2] += 8 * picture->pitches[2];		\
-	} while (0);							\
-	picture->v_offset += 16;					\
-	if (picture->v_offset > picture->limit_y) {			\
-	    if (mpeg2_cpu_state_restore)				\
-		mpeg2_cpu_state_restore (&cpu_state);			\
-	    return;							\
-	}								\
-	picture->offset = 0;						\
-    }									\
+#define NEXT_MACROBLOCK							    \
+do {									    \
+    picture->offset += 16;						    \
+    if (picture->offset == picture->coded_picture_width) {		    \
+	do { /* just so we can use the break statement */		    \
+	    if (picture->current_frame->proc_slice) {			    \
+		picture->current_frame->proc_slice (picture->current_frame, \
+						    picture->dest);	    \
+	    }								    \
+	    picture->dest[0] += 16 * picture->pitches[0];		    \
+	    picture->dest[1] += 8 * picture->pitches[1];		    \
+	    picture->dest[2] += 8 * picture->pitches[2];		    \
+	} while (0);							    \
+	picture->v_offset += 16;					    \
+	if (picture->v_offset > picture->limit_y) {			    \
+	    if (mpeg2_cpu_state_restore)				    \
+		mpeg2_cpu_state_restore (&cpu_state);			    \
+	    return;							    \
+	}								    \
+	picture->offset = 0;						    \
+    }									    \
 } while (0)
 
 static inline int slice_init (picture_t * picture, int code)
@@ -1520,9 +1534,7 @@ static inline int slice_init (picture_t * picture, int code)
 
     picture->v_offset = (code - 1) * 16;
     offset = (code - 1);
-    if (picture->current_frame->copy && picture->picture_coding_type == B_TYPE)
-	offset = 0;
-    else if (picture->picture_structure != FRAME_PICTURE)
+    if (picture->picture_structure != FRAME_PICTURE)
 	offset = 2 * offset;
 
     picture->dest[0] = picture->current_frame->base[0] + picture->pitches[0] * offset * 16;
@@ -1587,7 +1599,7 @@ static inline int slice_init (picture_t * picture, int code)
 
     while (picture->offset - picture->coded_picture_width >= 0) {
 	picture->offset -= picture->coded_picture_width;
-	if ((picture->current_frame->copy == NULL) ||
+	if ((picture->current_frame->proc_slice == NULL) ||
 	    (picture->picture_coding_type != B_TYPE)) {
 	    picture->dest[0] += 16 * picture->pitches[0];
 	    picture->dest[1] += 8 * picture->pitches[1];
