@@ -22,16 +22,6 @@
 #include <ctype.h>
 #include <dlfcn.h>
 
-struct modref_list_t;
-
-typedef struct modref_list_t
-{
-    WINE_MODREF* wm;
-    struct modref_list_t *next;
-    struct modref_list_t *prev;    
-}
-modref_list;
-
 
 //WINE_MODREF *local_wm=NULL;
 extern modref_list* local_wm;
@@ -43,7 +33,6 @@ extern void dump_exports(HMODULE hModule);
 /*---------------- END HACKS ---------------*/
 
 //char *extra_ld_library_path = "/usr/lib/win32";
-extern char* win32_def_path;
 
 struct elfdll_image
 {
