@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer_types.c,v 1.89 2004/11/30 00:40:59 tmattern Exp $
+ * $Id: buffer_types.c,v 1.89.2.3 2005/04/25 14:43:17 mroi Exp $
  *
  *
  * contents:
@@ -81,10 +81,11 @@ static video_db_t video_db[] = {
     meFOURCC('m', 'p', '4', 'v'),
     meFOURCC('M', '4', 'S', '2'),
     meFOURCC('m', '4', 's', '2'),
+    meFOURCC('F', 'M', 'P', '4'),
     0
   },
   BUF_VIDEO_MPEG4,
-  "ISO-MPEG4/OpenDivx format"
+  "ISO-MPEG4/OpenDivx"
 },
 {
   {
@@ -93,7 +94,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_XVID,
-  "XviD format"
+  "XviD"
 },
 {
   {
@@ -101,7 +102,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_DIVX5,
-  "DivX 5 format"
+  "DivX 5"
 },
 {
   {
@@ -109,7 +110,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_CINEPAK,
-  "Cinepak format"
+  "Cinepak"
 },
 {
   {
@@ -139,7 +140,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_MSMPEG4_V1,
-  "Microsoft MPEG-4 format v1"
+  "Microsoft MPEG-4 v1"
 },
 {
   {
@@ -152,7 +153,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_MSMPEG4_V2,
-  "Microsoft MPEG-4 format v2"
+  "Microsoft MPEG-4 v2"
 },
 {
   {
@@ -172,7 +173,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_MSMPEG4_V3,
-  "Microsoft MPEG-4 format v3"
+  "Microsoft MPEG-4 v3"
 },
 {
   {
@@ -188,13 +189,20 @@ static video_db_t video_db[] = {
     meFOURCC('d', 'm', 'b', '1'),
     meFOURCC('M', 'J', 'P', 'G'),
     meFOURCC('m', 'j', 'p', 'a'),
-    meFOURCC('m', 'j', 'p', 'b'),
     meFOURCC('A', 'V', 'R', 'n'),
     meFOURCC('A', 'V', 'D', 'J'),
     0
   },
   BUF_VIDEO_MJPEG,
-  "motion jpeg format"
+  "Motion JPEG"
+},
+{
+  {
+    meFOURCC('m', 'j', 'p', 'b'),
+    0
+  },
+  BUF_VIDEO_MJPEG_B,
+  "Motion JPEG B"
 },
 {
   {
@@ -203,7 +211,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV50,
-  "Indeo Video 5.0 format"
+  "Indeo Video 5.0"
 },
 {
   {
@@ -212,7 +220,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV41,
-  "Indeo Video 4.1 format"
+  "Indeo Video 4.1"
 },
 {
   {
@@ -221,7 +229,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV32,
-  "Indeo Video 3.2 format"
+  "Indeo Video 3.2"
 },
 {
   {
@@ -230,7 +238,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_IV31,
-  "Indeo Video 3.1 format"
+  "Indeo Video 3.1"
 },
 {
   {
@@ -238,7 +246,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_ATIVCR1,
-  "ATI VCR1 format"
+  "ATI VCR1"
 },
 {
   {
@@ -246,7 +254,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_ATIVCR2,
-  "ATI VCR2 format"
+  "ATI VCR2"
 },
 {
   {
@@ -258,7 +266,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_I263,
-  "I263 format"
+  "I263"
 },
 {
   {
@@ -288,7 +296,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_JPEG,
-  "jpeg"
+  "JPEG"
 },
 {
   {
@@ -348,7 +356,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_VP31,
-  "On2 VP3.1 Codec"
+  "On2 VP3.1"
 },
 {
   {
@@ -367,7 +375,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_H263,
-  "H263 format"
+  "H263"
 },
 {
   {
@@ -376,7 +384,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_CYUV,
-  "Creative YUV format"
+  "Creative YUV"
 },
 {
   {
@@ -393,7 +401,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_RPZA,
-  "Apple Quicktime Video (RPZA)"
+  "Apple Quicktime (RPZA)"
 },
 {
   {
@@ -432,7 +440,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_PGVV,
-  "Radius Studio Codec"
+  "Radius Studio"
 },
 {
   {
@@ -464,7 +472,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_GREY,
-  "Raw greyscale"
+  "Raw Greyscale"
 },
 {
   {
@@ -574,7 +582,7 @@ static video_db_t video_db[] = {
     0,
   },
   BUF_VIDEO_THEORA,
-  "OggTheora Video"
+  "Ogg Theora"
 },
 {
   {
@@ -582,7 +590,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_VP5,
-  "On2 VP5 Codec"
+  "On2 VP5"
 },
 {
   {
@@ -592,7 +600,7 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_VP6,
-  "On2 VP6 Codec"
+  "On2 VP6"
 },
 {
   {
@@ -637,6 +645,7 @@ static video_db_t video_db[] = {
 {
   {
     meFOURCC('a','v','c','1'),
+    meFOURCC('h','2','6','4'),
     0
   },
   BUF_VIDEO_H264,
@@ -707,6 +716,13 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_WMAV3,
   "Windows Media Audio v3"
 },
+{
+  {
+    0x163, 0
+  },
+  BUF_AUDIO_WMALL,
+  "Windows Media Audio Lossless"
+},
 {
   {
     0xA, 0
@@ -967,11 +983,18 @@ static audio_db_t audio_db[] = {
 },
 {
   {
-    0
+    meFOURCC('a', 'l', 'a', 'c'),
   },
   BUF_AUDIO_ALAC,
   "Apple Lossless Audio Codec"
 },
+{
+  {
+    0
+  },
+  BUF_AUDIO_MPC,
+  "Musepack"
+},
 { { 0 }, 0, "last entry" }
 };
 
