@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2003 the xine project
  *
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_overlay.h,v 1.8 2002/04/09 13:20:44 jcdutton Exp $
+ * $Id: video_overlay.h,v 1.19 2004/12/08 18:18:24 miguelfreitas Exp $
  *
  */
 
 #ifndef HAVE_VIDEO_OVERLAY_H
 #define HAVE_VIDEO_OVERLAY_H
 
-#include "xine_internal.h"
+#ifdef XINE_COMPILE
+#  include "xine_internal.h"
+#else
+#  include <xine/xine_internal.h>
+#endif
 
 #ifdef	__GNUC__
 #define CLUT_Y_CR_CB_INIT(_y,_cr,_cb)	{y: (_y), cr: (_cr), cb: (_cb)}
@@ -34,17 +38,13 @@
 
 #define MAX_OBJECTS   50
 #define MAX_EVENTS    50
-#define MAX_SHOWING    5
+#define MAX_SHOWING   16
 
-#define EVENT_NULL             0
-#define EVENT_SHOW_SPU         1
-#define EVENT_HIDE_SPU         2
-#define EVENT_SHOW_MENU        3
-#define EVENT_HIDE_MENU        4
-#define EVENT_MENU_BUTTON      5
-#define EVENT_DELETE_RESOURCE  6 /* Maybe release handle will do this */
-#define EVENT_SHOW_OSD         7 /* Not yet implemented */
-#define EVENT_FREE_HANDLE      8 /* Frees a handle, previous allocated via get_handle */
+#define OVERLAY_EVENT_NULL             0
+#define OVERLAY_EVENT_SHOW             1
+#define OVERLAY_EVENT_HIDE             2
+#define OVERLAY_EVENT_MENU_BUTTON      3
+#define OVERLAY_EVENT_FREE_HANDLE      8 /* Frees a handle, previous allocated via get_handle */
 
 /* number of colors in the overlay palette. Currently limited to 256
    at most, because some alphablend functions use an 8-bit index into
@@ -57,6 +57,9 @@ typedef struct vo_buttons_s {
                                      1:Button Valid, no auto_action,
                                      2:Button Valid, auto_action.
                                   */
+
+  /* The following clipping coordinates are relative to the left upper corner
+   * of the OVERLAY, not of the target FRAME. Please do not mix them up! */
   int32_t           clip_top;
   int32_t           clip_bottom;
   int32_t           clip_left;
@@ -67,10 +70,10 @@ typedef struct vo_buttons_s {
   int32_t           right; 
   uint32_t          select_color[OVL_PALETTE_SIZE];
   uint8_t           select_trans[OVL_PALETTE_SIZE];
-  xine_menu_event_t select_event;
+  xine_event_t      select_event;
   uint32_t          active_color[OVL_PALETTE_SIZE];
   uint8_t           active_trans[OVL_PALETTE_SIZE];
-  xine_menu_event_t active_event;
+  xine_event_t      active_event;
   int32_t           clip_rgb_clut;      /* true if clut was converted to rgb*/
                                         /* FIXME: Probably not needed ^^^ */
 } vo_buttons_t;
@@ -98,4 +101,6 @@ typedef struct video_overlay_event_s {
   video_overlay_object_t   object; /* The image data. */
 } video_overlay_event_t;
 
+video_overlay_manager_t *_x_video_overlay_new_manager(xine_t *);
+
 #endif
