@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.306 2004/12/20 21:22:21 mroi Exp $
+ * $Id: xine.c,v 1.306.2.2 2005/04/20 17:21:17 mroi Exp $
  */
 
 /*
@@ -273,28 +273,26 @@ static void stop_internal (xine_stream_t *stream) {
 
   lprintf ("status before = %d\n", stream->status);
 
-  if (stream->status == XINE_STATUS_STOP) {
+  if ( stream->status == XINE_STATUS_IDLE ||
+       stream->status == XINE_STATUS_STOP ) {
     _x_demux_control_end(stream, 0);
-    lprintf ("ignored\n");
-    return;
+    lprintf("ignored");
+  } else {
+    /* make sure we're not in "paused" state */
+    set_speed_internal (stream, XINE_FINE_SPEED_NORMAL);
+
+    /* Don't change status if we're quitting */
+    if (stream->status != XINE_STATUS_QUIT)
+      stream->status = XINE_STATUS_STOP;
   }
-
-  /* make sure we're not in "paused" state */
-  set_speed_internal (stream, XINE_FINE_SPEED_NORMAL);
-
-  /* Don't change status if we're quitting */
-  if (stream->status != XINE_STATUS_QUIT)
-    stream->status = XINE_STATUS_STOP;
-
   /*
    * stop demux
    */
-  lprintf ("stopping demux\n");
-  if (stream->demux_plugin) {
+  if (stream->demux_plugin && stream->demux_thread) {
+    lprintf ("stopping demux\n");
     _x_demux_stop_thread( stream );
-    lprintf ("stop thread done\n");
+    lprintf ("demux stopped\n");
   }
-  lprintf ("demux stopped\n");
   lprintf ("done\n");
 }
 
@@ -358,9 +356,8 @@ static void close_internal (xine_stream_t *stream) {
   stream->xine->port_ticket->release(stream->xine->port_ticket, 1);
   stream->ignore_speed_change = 0;
   
-  lprintf ("disposing demux\n");
   if (stream->demux_plugin) {
-    stream->demux_plugin->dispose (stream->demux_plugin);
+    _x_free_demux_plugin(stream, stream->demux_plugin);
     stream->demux_plugin = NULL;
   }
 
@@ -369,7 +366,7 @@ static void close_internal (xine_stream_t *stream) {
    */
 
   if (stream->input_plugin) {
-    stream->input_plugin->dispose(stream->input_plugin);
+    _x_free_input_plugin(stream, stream->input_plugin);
     stream->input_plugin = NULL;
   }
 
@@ -383,6 +380,9 @@ static void close_internal (xine_stream_t *stream) {
     _x_meta_info_reset(stream, i);
     _x_meta_info_public_reset(stream, i);
   }
+  stream->audio_track_map_entries = 0;
+  stream->spu_track_map_entries = 0;
+
 }
 
 void xine_close (xine_stream_t *stream) {
@@ -391,6 +391,16 @@ void xine_close (xine_stream_t *stream) {
 
   close_internal (stream);
 
+  /*
+   * set status to idle.
+   * not putting this into close_internal because it is also called
+   * by open_internal.
+   */
+
+  /* Don't change status if we're quitting */
+  if (stream->status != XINE_STATUS_QUIT)
+    stream->status = XINE_STATUS_IDLE;
+
   pthread_mutex_unlock (&stream->frontend_lock);
 }
 
@@ -467,7 +477,7 @@ xine_stream_t *xine_stream_new (xine_t *this,
   _x_extra_info_reset( stream->audio_decoder_extra_info );
 
   stream->xine                   = this;
-  stream->status                 = XINE_STATUS_STOP;
+  stream->status                 = XINE_STATUS_IDLE;
 
   stream->spu_decoder_plugin     = NULL;
   stream->spu_decoder_streamtype = -1;
@@ -614,8 +624,10 @@ void _x_flush_events_queues (xine_stream_t *stream) {
     pthread_mutex_lock (&queue->lock);
     pthread_mutex_unlock (&stream->event_queues_lock);
 
-    while (!xine_list_is_empty (queue->events)) {
-      pthread_cond_wait (&queue->events_processed, &queue->lock);
+    if (queue->listener_thread != NULL) {
+      while (!xine_list_is_empty (queue->events)) {
+        pthread_cond_wait (&queue->events_processed, &queue->lock);
+      }
     }
 
     pthread_mutex_unlock (&queue->lock);
@@ -633,7 +645,8 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
   if (!mrl) {
     xprintf (stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
     stream->err = XINE_ERROR_MALFORMED_MRL;
-    stream->status = XINE_STATUS_STOP;
+    if (stream->status != XINE_STATUS_IDLE)
+      stream->status = XINE_STATUS_STOP;
     return 0;
   }
 
@@ -718,7 +731,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	  if (!(stream->demux_plugin = _x_find_demux_plugin_by_name(stream, demux_name, stream->input_plugin))) {
 	    xine_log(stream->xine, XINE_LOG_MSG, _("xine: specified demuxer %s failed to start\n"), demux_name);
 	    stream->err = XINE_ERROR_NO_DEMUX_PLUGIN;
-	    stream->status = XINE_STATUS_STOP;
+	    stream->status = XINE_STATUS_IDLE;
 	    free(demux_name);
 	    return 0;
 	  }
@@ -729,7 +742,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	continue;
@@ -759,14 +772,14 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	  } else {
 	    xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error opening rip input plugin instance\n"));
 	    stream->err = XINE_ERROR_MALFORMED_MRL;
-	    stream->status = XINE_STATUS_STOP;
+	    stream->status = XINE_STATUS_IDLE;
 	    return 0;
 	  }
 
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	continue;
@@ -790,7 +803,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	  if (!(stream->demux_plugin = _x_find_demux_plugin_last_probe(stream, demux_name, stream->input_plugin))) {
 	    xine_log(stream->xine, XINE_LOG_MSG, _("xine: last_probed demuxer %s failed to start\n"), demux_name);
 	    stream->err = XINE_ERROR_NO_DEMUX_PLUGIN;
-	    stream->status = XINE_STATUS_STOP;
+	    stream->status = XINE_STATUS_IDLE;
 	    free(demux_name);
 	    return 0;
 	  }
@@ -802,7 +815,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	continue;
@@ -814,7 +827,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	xprintf (stream->xine, XINE_VERBOSITY_LOG, _("ignoring video\n"));
@@ -827,7 +840,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	xprintf (stream->xine, XINE_VERBOSITY_LOG, _("ignoring audio\n"));
@@ -840,7 +853,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	xprintf (stream->xine, XINE_VERBOSITY_LOG, _("ignoring subpicture\n"));
@@ -853,7 +866,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	xprintf (stream->xine, XINE_VERBOSITY_LOG, _("input cache plugin disabled\n"));
@@ -879,7 +892,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	continue;
@@ -904,7 +917,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	continue;
@@ -939,7 +952,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	} else {
 	  xprintf(stream->xine, XINE_VERBOSITY_LOG, _("xine: error while parsing mrl\n"));
 	  stream->err = XINE_ERROR_MALFORMED_MRL;
-	  stream->status = XINE_STATUS_STOP;
+	  stream->status = XINE_STATUS_IDLE;
 	  return 0;
 	}
 	continue;
@@ -970,7 +983,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 	      config_entry);
 	  }
           stream->err = XINE_ERROR_MALFORMED_MRL;
-          stream->status = XINE_STATUS_STOP;
+          stream->status = XINE_STATUS_IDLE;
 	  free(config_entry);
           return 0;
 	}
@@ -993,7 +1006,7 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
       xine_log (stream->xine, XINE_LOG_MSG, _("xine: couldn't find demux for >%s<\n"), mrl);
       stream->err = XINE_ERROR_NO_DEMUX_PLUGIN;
 
-      stream->status = XINE_STATUS_STOP;
+      stream->status = XINE_STATUS_IDLE;
 
       /* force the engine to unregister fifo callbacks */
       _x_demux_control_nop(stream, BUF_FLAG_END_STREAM);
@@ -1028,16 +1041,16 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
   if (stream->demux_plugin->get_status(stream->demux_plugin) != DEMUX_OK) {
     xine_log (stream->xine, XINE_LOG_MSG, _("xine: demuxer failed to start\n"));
 
-    stream->demux_plugin->dispose (stream->demux_plugin);
+    _x_free_demux_plugin(stream, stream->demux_plugin);
     stream->demux_plugin = NULL;
 
     xprintf (stream->xine, XINE_VERBOSITY_DEBUG, "demux disposed\n");
 
-    stream->input_plugin->dispose (stream->input_plugin);
+    _x_free_input_plugin(stream, stream->input_plugin);
     stream->input_plugin = NULL;
     stream->err = XINE_ERROR_NO_DEMUX_PLUGIN;
 
-    stream->status = XINE_STATUS_STOP;
+    stream->status = XINE_STATUS_IDLE;
 
     xprintf (stream->xine, XINE_VERBOSITY_DEBUG, "return from\n");
     return 0;
@@ -1045,6 +1058,8 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 
   _x_demux_control_headers_done (stream);
 
+  stream->status = XINE_STATUS_STOP;
+
   lprintf ("done\n");
   return 1;
 }
@@ -1342,7 +1357,7 @@ xine_t *xine_new (void) {
    * i18n
    */
 
-  bindtextdomain("xine-lib", XINE_LOCALEDIR);
+  bindtextdomain(XINE_TEXTDOMAIN, XINE_LOCALEDIR);
 #endif
 
   /*
